/*********************************************************************
 * This file is part of the PRAPI library.
 *
 * Copyright (C) 2001 Topi Menp
 * All rights reserved.
 *
 * This program is free software. You can redistribute and/or modify
 * it under the terms of the free software licence found in the
 * accompanying file "COPYING". The licence terms must always be
 * redistributed with this source file. The above copyright notice
 * must be reproduced in all modified and unmodified copies of this
 * source file.
 *
 * $Revision: 1.3 $
 *********************************************************************/

#ifndef _GAUSSIAN_H
#define _GAUSSIAN_H

#include <Matrix.h>

namespace prapi
{
	/**
	 * A class that contains methods for creating various forms of a 2D
	 * Gaussian function.
	 **/
	class Gaussian
	{
	public:
		/**
		 * Create a simple, symmetric gaussian function.
		 *
		 * @param radius calculate values between [-radius,radius]
		 * @param size the size of the output matrix (sampling frequency)
		 * @return a size x size matrix
		 * @param width the width of the gaussian (2*std<sup>2</sup>)
		 **/
		static util::Matrix<double> createSymmetricalGaussian(double radius, int size, double width=1);
	};
}

#endif
