/*********************************************************************
 * This file is part of the cpplibs suite.
 *
 * Copyright (C) 2001 Topi Menp
 * All rights reserved.
 *
 * This program is free software. You can redistribute and/or modify
 * it under the terms of the free software licence found in the
 * accompanying file "COPYING". The licence terms must always be
 * redistributed with this source file. The above copyright notice
 * must be reproduced in all modified and unmodified copies of this
 * source file.
 *
 * $Revision: 1.1 $
 *********************************************************************/

#ifndef _FUNCTION_H
#define _FUNCTION_H

#include "../Number.h"
#include "../SmartPtr.h"
#include <math.h>

namespace util { namespace ariel {

	class Function : public Computable
	{
	public:
		class Negate;
		class Complement;

		class MathFunction;

	protected:
		ComputationException createException(std::string name, const Computable& c) const
		{
			return ComputationException("Function::createException(string,const Computable&): " + name +
																	"(" + c.getClassName() + ") not defined.");
		}
	};

	class Function::Negate : public Function
	{
	public:
		SmartPtr<Computable> functionCall(const Computable& c) const throw (ComputationException&)
		{
			return c.negate();
		}
		Object* clone() const { return new Negate; }
	};

	class Function::Complement : public Function
	{
	public:
		SmartPtr<Computable> functionCall(const Computable& c) const throw (ComputationException&)
		{
			return c.complement();
		}
		Object* clone() const { return new Complement; }
	};

	class Function::MathFunction : public Function
	{
	public:
		typedef double (*FPointer)(double);
		
		MathFunction(FPointer function, std::string name) : _function(function), _strName(name) {}
		SmartPtr<Computable> functionCall(const Computable& c) const throw (ComputationException&)
		{
			const Number* number = dynamic_cast<const Number*>(&c);
			if (number)
				return SmartPtr<Computable>(new Number::Double(_function(number->doubleValue())));
			throw createException(_strName,c);
		}

		Object* clone() const { return new MathFunction(_function,_strName); }
	private:
		FPointer _function;
		std::string _strName;
	};
}}

#endif
